package com.persagy.account.controller;

import java.util.List;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Lists;
import com.persagy.account.manage.SaasRoleMenuHandler;
import com.persagy.account.pojo.dto.SaasMenu;
import com.persagy.account.pojo.dto.SaasProduct;
import com.persagy.account.pojo.vo.menu.SaasMenuCreateVO;
import com.persagy.account.pojo.vo.menu.SaasMenuDeleteVO;
import com.persagy.account.pojo.vo.menu.SaasMenuQueryVO;
import com.persagy.account.pojo.vo.menu.SaasMenuSortVO;
import com.persagy.account.service.ISaasMenuService;
import com.persagy.account.service.ISaasProductService;
import com.persagy.account.service.ISaasRoleMenuService;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.utils.ResponseResult;
import com.persagy.common.utils.ResponseResultUtil;
import com.persagy.common.utils.StringUtil;

import cn.hutool.core.collection.CollectionUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 菜单信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Api(tags = "菜单信息")
@RestController
@RequestMapping(value = "/menu", method = RequestMethod.POST)
public class SaasMenuController {
	
    @Autowired
    private ISaasMenuService saasMenuService;
    
    @Autowired
    private ISaasProductService saasProductService;
    
    @Autowired
    private SaasRoleMenuHandler saasRoleMenuHandler;
    
    @Autowired
    private ISaasRoleMenuService saasRoleMenuService;
    
    /**
     * 查询账号菜单功能权限树，根据账号ID做权限过滤，主要供账号登录后左侧展示使用
     */
    @ApiOperation(value = "查询账号菜单功能权限树")
    @RequestMapping(value = "querySaasMenuTree")
    public ResponseResult querySaasMenuTree(@RequestBody @Validated SaasMenuQueryVO queryVO) {
    	List<SaasMenu> menuTree = this.saasRoleMenuHandler.querySaasMenuTree(queryVO);
    	return ResponseResultUtil.successResult(menuTree);
    }
    
    /**
     * 查询顶级菜单，菜单配置使用
     */
    @ApiOperation(value = "查询顶级菜单")
    @RequestMapping(value = "querySaasMenuTop")
    public ResponseResult querySaasMenuTop(@RequestBody @Validated SaasMenuQueryVO queryVO) {
    	LambdaQueryWrapper<SaasMenu> queryWrapper = new SaasMenu.Builder().createQueryWrapper()
    			.appIdEq(queryVO.getAppId()).menuTypeEq(queryVO.getMenuType()).builderQueryWrapper();
    	queryWrapper.apply("(PARENT_ID IS NULL OR TRIM(PARENT_ID) = '')");
    	queryWrapper.orderByAsc(SaasMenu::getMenuSort);		// 默认菜单升序排序
    	List<SaasMenu> menuTree = this.saasMenuService.list(queryWrapper);
    	return ResponseResultUtil.successResult(menuTree);
    }
    
    /**
     * 查询菜单功能树，角色配置使用
     */
    @ApiOperation(value = "查询菜单功能树")
    @RequestMapping(value = "queryMenuFunTree")
    public ResponseResult queryMenuFunTree(@RequestBody @Validated SaasMenuQueryVO queryVO) {
    	if (StringUtil.isBlank(queryVO.getMenuType())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "菜单类型不可为空");
		}
    	// 这里按照超管的逻辑进行处理，因为不会做权限过滤
		LambdaQueryWrapper<SaasMenu> queryWrapper = new SaasMenu.Builder().createQueryWrapper().idEq(queryVO.getMenuId())
				.menuTypeEq(queryVO.getMenuType()).appIdEq(queryVO.getAppId()).builderQueryWrapper();
		queryWrapper.apply("(PARENT_ID IS NULL OR TRIM(PARENT_ID) = '')");
		queryWrapper.orderByAsc(SaasMenu::getMenuSort);		// 默认菜单升序排序
		List<SaasMenu> topList = this.saasMenuService.list(queryWrapper);
		if (CollectionUtil.isEmpty(topList)) {
			return ResponseResultUtil.successResult(Lists.newArrayList());
		}
		// 2.转为菜单树
		this.saasRoleMenuHandler.setMenuChildrens(null, queryVO.getGroupCode(), queryVO.getProjectId(), null, topList, true, !queryVO.isMenu());
    	return ResponseResultUtil.successResult(topList);
    }
    
    /**
     * 新增
     */
	@ApiOperation(value = "保存")
	@RequestMapping(value = "createSaasMenu")
    public ResponseResult createSaasMenu(@RequestBody @Validated SaasMenuCreateVO createVO) {
		if (StringUtil.isNotBlank(createVO.getMenuUrl()) && StringUtil.isEmpty(createVO.getUrlType())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "当菜单URL存在时，URL类型不可为空");
		}
		// 应用下菜单编码唯一
		ResponseResult validRequestParam = this.validRequestParam(createVO.getAppId(), createVO.getMenuType(), createVO.getMenuCode(), createVO.getMenuName(), null);
		if (validRequestParam != null) {
			return validRequestParam;
		}
		SaasMenu saasMenu = new SaasMenu();
		BeanUtils.copyProperties(createVO, saasMenu);
		saasMenu.setId(null);
		saasMenu.setUpdateUser(createVO.getAccountId());
		// 获取产品图标，设置进菜单图标
		if (StringUtil.isNotBlank(createVO.getProductId())) {
			SaasProduct saasProduct = this.saasProductService.getById(createVO.getProductId());
			if (saasProduct == null) {
				return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "不存在此产品");
			}
			saasMenu.setMenuIcon(saasProduct.getProductIcon());
		}
		boolean result = this.saasMenuService.createSaasMenu(saasMenu);
        return result ? ResponseResultUtil.successResult("保存成功", saasMenu.getId()) : ResponseResultUtil.errorResult("保存失败");
    }
    
    /**
     * 更新
     */
	@ApiOperation(value = "更新")
	@RequestMapping(value = "updateSaasMenu")
    public ResponseResult updateSaasMenu(@RequestBody @Validated SaasMenuCreateVO createVO) {
		if (StringUtil.isBlank(createVO.getId())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "菜单ID不可为空");
		}
		if (StringUtil.isNotBlank(createVO.getMenuUrl()) && StringUtil.isEmpty(createVO.getUrlType())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "当菜单URL存在时，URL类型不可为空");
		}
		// 验证菜单是否存在
		SaasMenu menu = this.saasMenuService.getById(createVO.getId());
		if (menu == null) {
			return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "不存在此菜单");
		}
		// 应用下菜单编码唯一
		ResponseResult validRequestParam = this.validRequestParam(createVO.getAppId(), createVO.getMenuType(), createVO.getMenuCode(), createVO.getMenuName(), createVO.getId());
		if (validRequestParam != null) {
			return validRequestParam;
		}
		// 验证是否允许菜单转为产品页
		if (StringUtil.isBlank(menu.getProductId()) && StringUtil.isNotBlank(createVO.getProductId())) {
			LambdaQueryWrapper<SaasMenu> queryWrapper = new SaasMenu.Builder().createQueryWrapper().parentIdEq(createVO.getId()).builderQueryWrapper();
			int count = this.saasMenuService.count(queryWrapper);
			if (count > 0) {
				return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "此菜单存在下级菜单，不允许转变为产品页");
			}
		}
		SaasMenu saasMenu = new SaasMenu();
		BeanUtils.copyProperties(createVO, saasMenu);
		saasMenu.setUpdateUser(createVO.getAccountId());
		// 获取产品图标，设置进菜单图标
		if (StringUtil.isNotBlank(createVO.getProductId())) {
			SaasProduct saasProduct = this.saasProductService.getById(createVO.getProductId());
			if (saasProduct == null) {
				return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "不存在此产品");
			}
			saasMenu.setMenuIcon(saasProduct.getProductIcon());
		}
		boolean result = this.saasMenuService.updateSaasMenu(saasMenu, menu.getProductId());
        return result ? ResponseResultUtil.successResult("更新成功") : ResponseResultUtil.errorResult("更新失败");
    }

    /**
     * 物理删除
     */
    @ApiOperation(value = "删除")
    @RequestMapping(value = "deleteSaasMenu")
    public ResponseResult deleteSaasMenu(@RequestBody @Validated SaasMenuDeleteVO deleteVO) {
    	// 只有此菜单不属于任何可用的角色才允许被删除
    	boolean result = this.saasRoleMenuService.isAllowDeleteMenu(deleteVO.getGroupCode(), deleteVO.getId());
    	if (!result) {
    		return ResponseResultUtil.errorResult(ResponseCode.A0402.getCode(), "该菜单已被使用，不允许被删除");
		}
    	SaasMenu saasMenu = this.saasMenuService.getById(deleteVO.getId());
    	if (saasMenu == null) {
    		return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "此菜单不存在");
		}
    	boolean updateProduct = false;
    	if (StringUtil.isNotBlank(saasMenu.getProductId())) {
			// 判断此产品是否仅被此菜单使用
    		LambdaQueryWrapper<SaasMenu> queryWrapper = new SaasMenu.Builder().createQueryWrapper().productId(saasMenu.getProductId()).builderQueryWrapper();
    		int count = this.saasMenuService.count(queryWrapper);
    		if (count < 2) {
    			updateProduct = true;
			}
		}
    	result = this.saasRoleMenuHandler.deleteSaasMenu(deleteVO.getId(), saasMenu.getProductId(), updateProduct);
        return result ? ResponseResultUtil.successResult("删除成功") : ResponseResultUtil.errorResult("删除失败");
    }
    
    /**
     * 菜单顺序调整
     */
    @ApiOperation(value = "菜单顺序调整")
    @RequestMapping(value = "sortSaasMenu")
    public ResponseResult sortSaasMenu(@RequestBody @Validated SaasMenuSortVO model) {
    	boolean result = this.saasMenuService.sortSaasMenu(model.getMenus());
        return result ? ResponseResultUtil.successResult("调整成功") : ResponseResultUtil.errorResult("调整失败");
    }
    
    /**
     * 
     * @return
     */
    private ResponseResult validRequestParam(String appId, String menuType, String menuCode, String menuName, String menuId) {
    	if (StringUtil.isBlank(appId)) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "应用ID，不可为空");
		}
		if (StringUtil.isNotBlank(menuCode)) {
			boolean validMenuCode = this.saasMenuService.validMenuCode(appId, menuType, menuCode, menuId);
			if (!validMenuCode) {
				return ResponseResultUtil.errorResult(ResponseCode.A0402.getCode(), "此编码已存在");
			}
		}
		if (StringUtil.isNotBlank(menuName)) {
			boolean validMenuName = this.saasMenuService.validMenuName(appId, menuType, menuName, menuId);
			if (!validMenuName) {
				return ResponseResultUtil.errorResult(ResponseCode.A0402.getCode(), "此名称已存在");
			}
		}
    	return null;
    }
}
